
#define IS_REGISTERED 256

#define MAX_DEMO      15    // days

#define PRODUCT_EXPIRED "This product is expired, call Dialsoft at (973) 335 - 3523!"
#define PRODUCT_REGISTERED "Thank you for registering!"
#define PRODUCT_DEMOMODE "Product expires in %d days!"

CHAR *n2hex(LONG n);
CHAR *usr2id(CHAR *uid);
SHORT mactcode(CHAR *check,CHAR *prod,ULONG prod_ver);

#define EXPIRED(x) \
   (cofdat(today())-cofdat(config.installdate) > x) || \
   (config.daycount > x) ||                            \
   (today() < config.installdate) ||                   \
   (today() < config.lastrun)


int check_gen(int prodcode);

/********
   Each new install of a product places an object into the bbsgen
   database of the BBS.  We imprint that object with the product
   code and the cofdat of the date installed.  Each time we
   boot up and check the demo/registered status, we check that cofdat
   date versus the current cofdat date and use this information
   to determine how many (if any) demo days are left.

   genobject must be a certain size and have a certain format.
   This format indicates that the object must be variable size
   and atleast UIDSIZ+MNMSIZ big.  After that, we can tack on
   as much stuff as we want.  In our case, we only use installed_cofdat

   Because prodcode has to be UIDSIZ, we place a string there
   that is the l2a rep of the product code.

*********/

typedef struct genobject {

   char prodcode[UIDSIZ];    // ascii rep of the product code
   char modnam[MNMSIZ];      // ascii rep of the module name (not used)
   int installed_cofdat;

}genobject;

// Prodcodes

// 1            Dialsoft Magic Keys
// 2            Dialdate
// 3            Web Spinner
// 4            Dialsoft Who's Online Active HTML
// 5            Dialsoft Web Blaster PRo code for Jack Alvrus
// 6            Dialsoft Graffiti Wall Active HTML
// 7            Dialsoft ActiveHTML Signature Editor for Phil Henning
// 8            Global Graffiti Wall modified by WDJ
// 9            Chit Chatters for 3.0 NT
// 10           Dialsoft Control, new version (v??)
// 11           Asp authetication module
// 12           Oltima WG2.00
// 13           Oltima WG3.20
// 14           ActiveHTML web blaster PRO
// 15           Dialnag WG3.20
// 16           Control, version 4.0
/****************************************************************

   Dialsoft activation code checker.

   This file is #included into each module that I modify for
   Dialsoft so we have a common and centralized activation code
   scheme.  This comes mostly into play for the Micromagic line
   which is huge and needs to be converted to the Dialsoft way.

   Demo days are indicated by a data item placed into bbsgen.dat.

   Returns:

   >1 (max of 15) - Indicates that the actcode was wrong
                    but you have this many demo days left.

   0              - Indicates that the actcode was wrong
                    and the demo has expired.

   IS_REGISTERED  - #defined above but it means that the activation
                    code was legitimate

******************************************************************/


int dialact(char *actcode, char *bbsreg, int prodcode) {

   char compare[8];
   int retval;

   // First things first, switch on the PRODCODE and check
   // the bbsreg versus the actcode

   switch(prodcode) {
         case 1:   // Dialsoft Magic Keys
              compare[0] = bbsreg[2] + bbsreg[2] - 32;
              compare[1] = bbsreg[3] + bbsreg[3] - 31;
              compare[2] = bbsreg[1] + bbsreg[2] - 33;
              compare[3] = bbsreg[2] + bbsreg[1] - 34;
              compare[4] = bbsreg[4] + bbsreg[5] - 32;
              compare[5] = compare[3] + bbsreg[6] - 30;
              compare[6] = compare[2] + bbsreg[2] - 30;
              compare[7] = 0;
              break;
         case 2:   // Dialsoft Magic Keys
              compare[0] = bbsreg[1] + bbsreg[1] - 32;
              compare[1] = bbsreg[3] + bbsreg[5] - 32;
              compare[2] = bbsreg[2] + bbsreg[4] - 33;
              compare[3] = bbsreg[3] + bbsreg[2] - 31;
              compare[4] = bbsreg[5] + bbsreg[3] - 31;
              compare[5] = compare[3] + bbsreg[2] - 30;
              compare[6] = compare[2] + bbsreg[4] - 30;
              compare[7] = 0;
              break;
         case 3:   // Dialsoft Web Spinner
              compare[0] = bbsreg[2] + bbsreg[1] - 31;
              compare[1] = bbsreg[5] + bbsreg[3] - 34;
              compare[2] = bbsreg[3] + bbsreg[1] - 33;
              compare[3] = bbsreg[2] + bbsreg[6] - 33;
              compare[4] = bbsreg[1] + bbsreg[7] - 31;
              compare[5] = compare[1] + bbsreg[6] - 32;
              compare[6] = compare[5] + bbsreg[1] - 30;
              compare[7] = 0;
              break;
         case 4:   // Dialsoft who's online active html
              compare[0] = bbsreg[5] + bbsreg[0] - 30;
              compare[1] = bbsreg[5] + bbsreg[1] - 32;
              compare[2] = bbsreg[3] + bbsreg[7] - 32;
              compare[3] = bbsreg[1] + bbsreg[2] - 31;
              compare[4] = bbsreg[3] + bbsreg[3] - 30;
              compare[5] = compare[4] + bbsreg[1] - 31;
              compare[6] = compare[2] + bbsreg[3] - 31;
              compare[7] = 0;
              break;
         case 5:   // Dialsoft Web Blaster Pro for Jack
              compare[0] = bbsreg[2] + bbsreg[1] - 31;
              compare[1] = bbsreg[1] + bbsreg[0] - 32;
              compare[2] = bbsreg[3] + bbsreg[1] - 32;
              compare[3] = bbsreg[2] + bbsreg[3] - 32;
              compare[4] = bbsreg[1] + bbsreg[1] - 32;
              compare[5] = compare[3] + bbsreg[2] - 31;
              compare[6] = compare[1] + bbsreg[3] - 32;
              compare[7] = 0;
              break;
         case 8:
         case 6:   // Dialsoft ActiveH Graf WalL and Global Graffiti
              compare[0] = bbsreg[1] + bbsreg[2] - 31;
              compare[1] = bbsreg[1] + bbsreg[3] - 32;
              compare[2] = bbsreg[1] + bbsreg[6] - 32;
              compare[3] = bbsreg[2] + bbsreg[7] - 33;
              compare[4] = bbsreg[3] + bbsreg[6] - 32;
              compare[5] = compare[1] + bbsreg[1] - 31;
              compare[6] = compare[5] + bbsreg[2] - 33;
              compare[7] = 0;
              break;
         case 7:   // Dialsoft ActiveH Sig Editor
              compare[0] = bbsreg[2] + bbsreg[3] - 30;
              compare[1] = bbsreg[2] + bbsreg[1] - 30;
              compare[2] = bbsreg[2] + bbsreg[2] - 30;
              compare[3] = bbsreg[3] + bbsreg[3] - 30;
              compare[4] = bbsreg[1] + bbsreg[2] - 31;
              compare[5] = compare[1] + bbsreg[4] - 30;
              compare[6] = compare[2] + bbsreg[3] - 31;
              compare[7] = 0;
              break;
         case 9:   // Chit Chatters 3.0 NT
              compare[0] = bbsreg[2] + bbsreg[1] - 31;
              compare[1] = bbsreg[4] + bbsreg[3] - 30;
              compare[2] = bbsreg[3] + bbsreg[2] - 29;
              compare[3] = bbsreg[1] + bbsreg[2] - 31;
              compare[4] = bbsreg[2] + bbsreg[3] - 34;
              compare[5] = compare[1] + bbsreg[1] - 33;
              compare[6] = compare[4] + bbsreg[3] - 30;
              compare[7] = 0;
              break;
         case 10:   // Control
              compare[0] = bbsreg[3] + bbsreg[4] - 32;
              compare[1] = bbsreg[2] + bbsreg[4] - 33;
              compare[2] = bbsreg[4] + bbsreg[1] - 33;
              compare[3] = bbsreg[5] + bbsreg[2] - 31;
              compare[4] = bbsreg[3] + bbsreg[3] - 31;
              compare[5] = compare[1] + bbsreg[1] - 33;
              compare[6] = compare[1] + bbsreg[1] - 31;
              compare[7] = 0;
              break;
         case 11:   // Asp authentication module
              compare[0] = bbsreg[4] + bbsreg[2] - 32;
              compare[1] = bbsreg[4] + bbsreg[1] - 33;
              compare[2] = bbsreg[3] + bbsreg[6] - 31;
              compare[3] = bbsreg[1] + bbsreg[5] - 31;
              compare[4] = bbsreg[1] + bbsreg[1] - 31;
              compare[5] = compare[3] + bbsreg[2] - 33;
              compare[6] = compare[4] + bbsreg[2] - 31;
              compare[7] = 0;
              break;
         case 12:   // Oltima WG2.0
              compare[0] = bbsreg[1] + bbsreg[2] - 34;
              compare[1] = bbsreg[2] + bbsreg[1] - 33;
              compare[2] = bbsreg[5] + bbsreg[3] - 31;
              compare[3] = bbsreg[6] + bbsreg[4] - 31;
              compare[4] = bbsreg[1] + bbsreg[1] - 30;
              compare[5] = compare[3] + bbsreg[2] - 33;
              compare[6] = compare[4] + bbsreg[2] - 30;
              compare[7] = 0;
              break;
         case 13:   // Oltima WG3.20
              compare[0] = bbsreg[1] + bbsreg[1] - 32;
              compare[1] = bbsreg[2] + bbsreg[1] - 33;
              compare[2] = bbsreg[3] + bbsreg[1] - 33;
              compare[3] = bbsreg[1] + bbsreg[5] - 31;
              compare[4] = bbsreg[1] + bbsreg[1] - 31;
              compare[5] = compare[4] + bbsreg[2] - 33;
              compare[6] = compare[4] + bbsreg[3] - 33;
              compare[7] = 0;
              break;
         case 14:   // Web Blaster Pro ActiveHTML version
              compare[0] = bbsreg[1] + bbsreg[1] - 32;
              compare[1] = bbsreg[1] + bbsreg[2] - 34;
              compare[2] = bbsreg[3] + bbsreg[3] - 31;
              compare[3] = bbsreg[1] + bbsreg[5] - 31;
              compare[4] = bbsreg[2] + bbsreg[1] - 34;
              compare[5] = compare[3] + bbsreg[2] - 33;
              compare[6] = compare[1] + bbsreg[3] - 33;
              compare[7] = 0;
              break;
         case 15:   // Dial nag
              compare[0] = bbsreg[2] + bbsreg[1] - 32;
              compare[1] = bbsreg[3] + bbsreg[3] - 34;
              compare[2] = bbsreg[1] + bbsreg[3] - 33;
              compare[3] = bbsreg[3] + bbsreg[1] - 31;
              compare[4] = bbsreg[4] + bbsreg[2] - 34;
              compare[5] = compare[3] + bbsreg[2] - 32;
              compare[6] = compare[2] + bbsreg[2] - 33;
              compare[7] = 0;
              break;
         case 16:   // Control version 4.0
              compare[0] = bbsreg[2] + bbsreg[0] - 31;
              compare[1] = bbsreg[2] + bbsreg[3] - 34;
              compare[2] = bbsreg[3] + bbsreg[2] - 30;
              compare[3] = bbsreg[4] + bbsreg[1] - 31;
              compare[4] = bbsreg[4] + bbsreg[2] - 34;
              compare[5] = compare[2] + bbsreg[4] - 32;
              compare[6] = compare[4] + bbsreg[2] - 33;
              compare[7] = 0;
              break;

   }

   if(sameas(compare,actcode)) {   // it matched,
       retval = IS_REGISTERED;
   }
   else {                           // no match, now check demo days!
       retval = check_gen(prodcode);
   }

   return retval;

}

/*
   Check to see if a gen object exists in the database for this
   product code and if so how many days are left on it.

   Returns:
       1->15  Demo days left
       0      Demo expired

*/

int check_gen(int prodcode) {

   char buffer[10];
   int retval;
   DFAFILE *ptr;
   genobject temp;

   ptr = dfaOpen("wgsgen2.dat",GENSIZ,NULL);
   dfaSetBlk(ptr);

   sprintf(buffer,"%d",prodcode);
   if(!dfaAcqEQ(&temp,buffer,1)) {   // not there, lets make one!
     strcpy(temp.modnam,buffer);
     strcpy(temp.prodcode,"D");
     temp.installed_cofdat = cofdat(today());
     dfaInsertV(&temp,sizeof(genobject));
     retval = 15;
   }
   else {
       retval = cofdat(today()) - temp.installed_cofdat;

       if(retval>15) {
          retval = 0;
       }
       else {
          retval = 15 - retval;
       }
   }

   return retval;
}



SHORT
mactcode(CHAR *check,CHAR *prod,ULONG prod_ver)
{
    CHAR prod_code[9];
    ULONG result,sum = 0;
    ULONG y,z,k;
    SHORT x;
	CHAR temp[100];
	char *tt;

	stlcpy(prod_code,prod,9);
	k=1;

	for( x=0; x<8; x++ ) {
		  y = (LONG)prod_code[x];
          z = (LONG)bturno[x] + prod_ver;
		  k=(x+y+z)*k*k;
		  prod_ver=prod_ver*k;
		  result = (y*y*(y+k) + z*z*(z+k) + x*x*(x+k)+k) + prod_ver;
          sum += result;
    }
	

	stzcpy( temp, l2as(sum), 100 );
	result=strcmp(usr2id(temp),check);
	if (result==0) return 1;
	else return 0;
}


CHAR *
usr2id(CHAR *uid)
{
 	INT i;
	CHAR tmpid[100];
	strcpy(tmpid,n2hex(uid[0]));
	//shocst("!","%c %s",uid[0],tmpid);
	for (i=1;i<=strlen(uid);i++) {
		strcat(tmpid,n2hex(uid[i]));
		//shocst("!","%c %s",uid[i],tmpid);
	}

	return strdup(tmpid);
}

CHAR *
n2hex(LONG n)
{
	CHAR dig[]="aAbBcCdDeEfFgGhHiIjJkKlLmMnNoOpPqQrRsStTuUvVwWxXyYzZ1234567890!@#$%^&*()_+-={}[]:;',./?|";
	CHAR myHEX[1000];
	LONG d,e;
	INT k,base;

	base=strlen(dig);

	k=0;
	e=0;

	if (n>=base)
	{
		while(n>=base)
		{
			d=fmod(n,base);
			e=(n-d)/base;
			n=e;
			myHEX[k]=dig[d];
			k++;
		}

		myHEX[k]=dig[e];
		myHEX[k+1]='\0';
		strrev(myHEX);
	}
	else
	{
		myHEX[k]=dig[n];
		myHEX[k+1]='\0';
	}

	// shocst("!","%lu %s",n,strdup(myHEX));
	return strdup(myHEX);
}

